@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/internal.h
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.0   2010/11/11
+ * Version: 1.8.0+   2011/02/07
  */
 
 #ifndef _SECURITY_CCSECURITY_INTERNAL_H
@@ -336,8 +336,7 @@ static inline void list_add_rcu(struct list_head *new, struct list_head *head)
 
 #ifndef CONFIG_CCSECURITY
 #define CONFIG_CCSECURITY
-#define CONFIG_CCSECURITY_MAX_GRANT_LOG    1024
-#define CONFIG_CCSECURITY_MAX_REJECT_LOG   1024
+#define CONFIG_CCSECURITY_MAX_AUDIT_LOG    1024
 #define CONFIG_CCSECURITY_MAX_ACCEPT_ENTRY 2048
 #define CONFIG_CCSECURITY_DEFAULT_LOADER "/sbin/ccs-init"
 #define CONFIG_CCSECURITY_ALTERNATIVE_TRIGGER "/sbin/ccs-start"
@@ -347,6 +346,9 @@ static inline void list_add_rcu(struct list_head *new, struct list_head *head)
 #define CONFIG_CCSECURITY_USE_EXTERNAL_TASK_SECURITY
 #endif
 #include "ccsecurity.h"
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#error This module supports only 2.6.0 and later kernels.
+#endif
 #ifndef CONFIG_SECURITY
 #error You must choose CONFIG_SECURITY=y for building this module.
 #endif
@@ -359,9 +361,6 @@ static inline void list_add_rcu(struct list_head *new, struct list_head *head)
 #ifndef CONFIG_MODULES
 #error You must choose CONFIG_MODULES=y for building this module.
 #endif
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
-#error This version is not supported because I cannot resolve vfsmount_lock .
-#endif
 
 /* Enumeration definition for internal use. */
 
@@ -679,8 +678,7 @@ enum ccs_policy_stat_type {
 
 /* Index numbers for profile's PREFERENCE values. */
 enum ccs_pref_index {
-	CCS_PREF_MAX_GRANT_LOG,
-	CCS_PREF_MAX_REJECT_LOG,
+	CCS_PREF_MAX_AUDIT_LOG,
 	CCS_PREF_MAX_LEARNING_ENTRY,
 	CCS_PREF_ENFORCING_PENALTY,
 	CCS_MAX_PREF
@@ -694,8 +692,7 @@ enum ccs_proc_interface_index {
 	CCS_PROCESS_STATUS,
 	CCS_MEMINFO,
 	CCS_STAT,
-	CCS_GRANTLOG,
-	CCS_REJECTLOG,
+	CCS_AUDIT,
 	CCS_VERSION,
 	CCS_PROFILE,
 	CCS_QUERY,
@@ -775,12 +772,10 @@ enum ccs_value_type {
 #define CCS_OPEN_FOR_IOCTL_ONLY           2
 /* Current thread is doing do_execve() ? */
 #define CCS_TASK_IS_IN_EXECVE             4
-/* Current thread should not be carried sleep penalty? */
-#define CCS_DONT_SLEEP_ON_ENFORCE_ERROR   8
 /* Current thread is running as an execute handler program? */
-#define CCS_TASK_IS_EXECUTE_HANDLER      16
-/* Curren thread is allowed to modify policy via /proc/ccs/ interface? */
-#define CCS_TASK_IS_MANAGER              32
+#define CCS_TASK_IS_EXECUTE_HANDLER       8
+/* Current thread is allowed to modify policy via /proc/ccs/ interface? */
+#define CCS_TASK_IS_MANAGER              16
 
 /*
  * Retry this request. Returned by ccs_supervisor() if policy violation has
@@ -1059,6 +1054,8 @@ struct ccs_request_info {
 	} param;
 	u8 param_type; /* One of values in "enum ccs_acl_entry_type_index". */
 	bool granted; /* True if granted, false otherwise. */
+	/* True if current thread should not be carried sleep penalty. */
+	bool dont_sleep_on_enforce_error;
 	/*
 	 * For updating current->ccs_domain_info at ccs_update_task_domain().
 	 * Initialized to NULL at ccs_init_request_info().
@@ -1449,6 +1446,7 @@ char *ccs_init_log(struct ccs_request_info *r, int len, const char *fmt,
 		   va_list args);
 char *ccs_read_token(struct ccs_acl_param *param);
 char *ccs_realpath_from_path(struct path *path);
+const char *ccs_yesno(const unsigned int value);
 const char *ccs_get_exe(void);
 const struct ccs_path_info *ccs_compare_name_union
 (const struct ccs_path_info *name, const struct ccs_name_union *ptr);
@@ -1515,8 +1513,6 @@ void ccs_check_acl(struct ccs_request_info *r,
 void ccs_convert_time(time_t time, struct ccs_time *p);
 void ccs_fill_path_info(struct ccs_path_info *ptr);
 void ccs_get_attributes(struct ccs_obj_info *obj);
-void ccs_io_printf(struct ccs_io_buffer *head, const char *fmt, ...)
-	__attribute__ ((format(printf, 2, 3)));
 void ccs_memory_free(const void *ptr, size_t size);
 void ccs_normalize_line(unsigned char *buffer);
 void ccs_print_ipv4(char *buffer, const int buffer_len, const u32 min_ip,
@@ -1943,7 +1939,9 @@ extern struct ccsecurity_exports ccsecurity_exports;
 struct ccs_security {
 	struct list_head list;
 	const struct task_struct *task; /* Maybe NULL. */
-	const struct cred *cred;  /* Maybe NULL. */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
+	const struct cred *cred; /* Maybe NULL. */
+#endif
 	struct ccs_domain_info *ccs_domain_info;
 	u32 ccs_flags;
 	struct ccs_execve *ee; /* Maybe NULL. */
@@ -1974,7 +1972,7 @@ static inline struct ccs_security *ccs_current_security(void)
 }
 
 /**
- * ccs_task_security - Get "struct ccs_security" for specified thread.
+ * ccs_task_domain - Get "struct ccs_domain_info" for specified thread.
  *
  * @task: Pointer to "struct task_struct".
  *
@@ -2016,7 +2014,7 @@ static inline u32 ccs_task_flags(struct task_struct *task)
 }
 
 /**
- * ccs_task_flags - Get flags for current thread.
+ * ccs_current_flags - Get flags for current thread.
  *
  * Returns flags for current thread.
  */
@@ -2059,7 +2057,7 @@ static inline struct ccs_security *ccs_current_security(void)
 }
 
 /**
- * ccs_task_security - Get "struct ccs_security" for specified thread.
+ * ccs_task_domain - Get "struct ccs_domain_info" for specified thread.
  *
  * @task: Pointer to "struct task_struct".
  *
@@ -2101,7 +2099,7 @@ static inline u32 ccs_task_flags(struct task_struct *task)
 }
 
 /**
- * ccs_task_flags - Get flags for current thread.
+ * ccs_current_flags - Get flags for current thread.
  *
  * Returns flags for current thread.
  */
