@@ -3,23 +3,9 @@
  *
  * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.8.0-pre   2010/10/18
- *
- * This file is applicable to both 2.4.30 and 2.6.11 and later.
- * See README.ccs for ChangeLog.
- *
+ * Version: 1.8.0+   2010/12/31
  */
 
-#include <linux/net.h>
-#include <linux/inet.h>
-#include <linux/in.h>
-#include <linux/in6.h>
-#include <linux/un.h>
-#include <net/sock.h>
-#include <net/af_unix.h>
-#include <net/ip.h>
-#include <net/ipv6.h>
-#include <net/udp.h>
 #include "internal.h"
 
 /* Structure for holding inet domain socket's address. */
@@ -63,65 +49,6 @@ const char * const ccs_socket_keyword[CCS_MAX_NETWORK_OPERATION] = {
 	[CCS_NETWORK_RECV]    = "recv",
 };
 
-/**
- * ccs_audit_net_log - Audit network log.
- *
- * @r:         Pointer to "struct ccs_request_info".
- * @family:    Name of socket family ("inet" or "unix").
- * @protocol:  Name of protocol in @family.
- * @operation: Name of socket operation.
- * @address:   Name of address.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int ccs_audit_net_log(struct ccs_request_info *r, const char *family,
-			     const u8 protocol, const u8 operation,
-			     const char *address)
-{
-	return ccs_supervisor(r, "network %s %s %s %s\n", family,
-			      ccs_proto_keyword[protocol],
-			      ccs_socket_keyword[operation], address);
-}
-
-/**
- * ccs_audit_inet_log - Audit INET network log.
- *
- * @r: Pointer to "struct ccs_request_info".
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int ccs_audit_inet_log(struct ccs_request_info *r)
-{
-	char buf[128];
-	int len;
-	const u32 *address = r->param.inet_network.address;
-	if (r->param.inet_network.is_ipv6)
-		ccs_print_ipv6(buf, sizeof(buf), (const struct in6_addr *)
-			       address, (const struct in6_addr *) address);
-	else
-		ccs_print_ipv4(buf, sizeof(buf), r->param.inet_network.ip,
-			       r->param.inet_network.ip);
-	len = strlen(buf);
-	snprintf(buf + len, sizeof(buf) - len, " %u",
-		 r->param.inet_network.port);
-	return ccs_audit_net_log(r, "inet", r->param.inet_network.protocol,
-				 r->param.inet_network.operation, buf);
-}
-
-/**
- * ccs_audit_unix_log - Audit UNIX network log.
- *
- * @r: Pointer to "struct ccs_request_info".
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int ccs_audit_unix_log(struct ccs_request_info *r)
-{
-	return ccs_audit_net_log(r, "unix", r->param.unix_network.protocol,
-				 r->param.unix_network.operation,
-				 r->param.unix_network.address->name);
-}
-
 /**
  * ccs_parse_ip_address - Parse an IP address.
  *
@@ -168,6 +95,24 @@ int ccs_parse_ip_address(char *address, u16 *min, u16 *max)
 	return CCS_IP_ADDRESS_TYPE_ADDRESS_GROUP;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 30)
+#if defined(__LITTLE_ENDIAN)
+#define HIPQUAD(addr)				\
+	((unsigned char *)&addr)[3],		\
+		((unsigned char *)&addr)[2],	\
+		((unsigned char *)&addr)[1],	\
+		((unsigned char *)&addr)[0]
+#elif defined(__BIG_ENDIAN)
+#define HIPQUAD(addr)				\
+	((unsigned char *)&addr)[0],		\
+		((unsigned char *)&addr)[1],	\
+		((unsigned char *)&addr)[2],	\
+		((unsigned char *)&addr)[3]
+#else
+#error "Please fix asm/byteorder.h"
+#endif /* __LITTLE_ENDIAN */
+#endif
+
 /**
  * ccs_print_ipv4 - Print an IPv4 address.
  *
@@ -188,11 +133,13 @@ void ccs_print_ipv4(char *buffer, const int buffer_len,
 }
 
 #if !defined(NIP6)
+
 #define NIP6(addr)							\
 	ntohs((addr).s6_addr16[0]), ntohs((addr).s6_addr16[1]),		\
 		ntohs((addr).s6_addr16[2]), ntohs((addr).s6_addr16[3]), \
 		ntohs((addr).s6_addr16[4]), ntohs((addr).s6_addr16[5]), \
 		ntohs((addr).s6_addr16[6]), ntohs((addr).s6_addr16[7])
+
 #endif
 
 /**
@@ -216,61 +163,6 @@ void ccs_print_ipv6(char *buffer, const int buffer_len,
 		 NIP6(*max_ip));
 }
 
-/**
- * ccs_check_inet_acl - Check permission for inet domain socket operation.
- *
- * @r:   Pointer to "struct ccs_request_info".
- * @ptr: Pointer to "struct ccs_acl_info".
- *
- * Returns true if granted, false otherwise.
- */
-static bool ccs_check_inet_acl(struct ccs_request_info *r,
-			       const struct ccs_acl_info *ptr)
-{
-	const struct ccs_inet_acl *acl = container_of(ptr, typeof(*acl), head);
-	bool ret;
-	if (!(acl->perm & (1 << r->param.inet_network.operation)) ||
-	    !ccs_compare_number_union(r->param.inet_network.port, &acl->port))
-		return false;
-	switch (acl->address_type) {
-	case CCS_IP_ADDRESS_TYPE_ADDRESS_GROUP:
-		ret = ccs_address_matches_group(r->param.inet_network.is_ipv6,
-						r->param.inet_network.address,
-						acl->address.group);
-		break;
-	case CCS_IP_ADDRESS_TYPE_IPv4:
-		ret = !r->param.inet_network.is_ipv6 &&
-			acl->address.ipv4.min <= r->param.inet_network.ip &&
-			r->param.inet_network.ip <= acl->address.ipv4.max;
-		break;
-	default:
-		ret = r->param.inet_network.is_ipv6 &&
-			memcmp(acl->address.ipv6.min,
-			       r->param.inet_network.address, 16) <= 0 &&
-			memcmp(r->param.inet_network.address,
-			       acl->address.ipv6.max, 16) <= 0;
-		break;
-	}
-	return ret;
-}
-
-/**
- * ccs_check_unix_acl - Check permission for unix domain socket operation.
- *
- * @r:   Pointer to "struct ccs_request_info".
- * @ptr: Pointer to "struct ccs_acl_info".
- *
- * Returns true if granted, false otherwise.
- */
-static bool ccs_check_unix_acl(struct ccs_request_info *r,
-			       const struct ccs_acl_info *ptr)
-{
-	const struct ccs_unix_acl *acl = container_of(ptr, typeof(*acl), head);
-	return (acl->perm & (1 << r->param.unix_network.operation)) &&
-		ccs_compare_name_union(r->param.unix_network.address,
-				       &acl->name);
-}
-
 /*
  * Mapping table from "enum ccs_network_acl_index" to "enum ccs_mac_index" for
  * inet domain socket.
@@ -363,7 +255,7 @@ static bool ccs_same_unix_acl(const struct ccs_acl_info *a,
  *
  * @a:         Pointer to "struct ccs_acl_info".
  * @b:         Pointer to "struct ccs_acl_info".
- * @is_delete: True for @a &= ~@b, false for @a |= @b . 
+ * @is_delete: True for @a &= ~@b, false for @a |= @b. 
  *
  * Returns true if @a is empty, false otherwise.
  */
@@ -386,7 +278,7 @@ static bool ccs_merge_inet_acl(struct ccs_acl_info *a, struct ccs_acl_info *b,
  *
  * @a:         Pointer to "struct ccs_acl_info".
  * @b:         Pointer to "struct ccs_acl_info".
- * @is_delete: True for @a &= ~@b, false for @a |= @b . 
+ * @is_delete: True for @a &= ~@b, false for @a |= @b. 
  *
  * Returns true if @a is empty, false otherwise.
  */
@@ -504,6 +396,8 @@ int ccs_write_unix_network(struct ccs_acl_param *param)
 
 /**
  * ccs_network_init - Dummy initialize function for CONFIG_NET=n case.
+ *
+ * Returns nothing.
  */
 void __init ccs_network_init(void)
 {
@@ -511,6 +405,120 @@ void __init ccs_network_init(void)
 
 #else
 
+/**
+ * ccs_audit_net_log - Audit network log.
+ *
+ * @r:         Pointer to "struct ccs_request_info".
+ * @family:    Name of socket family ("inet" or "unix").
+ * @protocol:  Name of protocol in @family.
+ * @operation: Name of socket operation.
+ * @address:   Name of address.
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int ccs_audit_net_log(struct ccs_request_info *r, const char *family,
+			     const u8 protocol, const u8 operation,
+			     const char *address)
+{
+	return ccs_supervisor(r, "network %s %s %s %s\n", family,
+			      ccs_proto_keyword[protocol],
+			      ccs_socket_keyword[operation], address);
+}
+
+/**
+ * ccs_audit_inet_log - Audit INET network log.
+ *
+ * @r: Pointer to "struct ccs_request_info".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int ccs_audit_inet_log(struct ccs_request_info *r)
+{
+	char buf[128];
+	int len;
+	const u32 *address = r->param.inet_network.address;
+	if (r->param.inet_network.is_ipv6)
+		ccs_print_ipv6(buf, sizeof(buf), (const struct in6_addr *)
+			       address, (const struct in6_addr *) address);
+	else
+		ccs_print_ipv4(buf, sizeof(buf), r->param.inet_network.ip,
+			       r->param.inet_network.ip);
+	len = strlen(buf);
+	snprintf(buf + len, sizeof(buf) - len, " %u",
+		 r->param.inet_network.port);
+	return ccs_audit_net_log(r, "inet", r->param.inet_network.protocol,
+				 r->param.inet_network.operation, buf);
+}
+
+/**
+ * ccs_audit_unix_log - Audit UNIX network log.
+ *
+ * @r: Pointer to "struct ccs_request_info".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int ccs_audit_unix_log(struct ccs_request_info *r)
+{
+	return ccs_audit_net_log(r, "unix", r->param.unix_network.protocol,
+				 r->param.unix_network.operation,
+				 r->param.unix_network.address->name);
+}
+
+/**
+ * ccs_check_inet_acl - Check permission for inet domain socket operation.
+ *
+ * @r:   Pointer to "struct ccs_request_info".
+ * @ptr: Pointer to "struct ccs_acl_info".
+ *
+ * Returns true if granted, false otherwise.
+ */
+static bool ccs_check_inet_acl(struct ccs_request_info *r,
+			       const struct ccs_acl_info *ptr)
+{
+	const struct ccs_inet_acl *acl = container_of(ptr, typeof(*acl), head);
+	bool ret;
+	if (!(acl->perm & (1 << r->param.inet_network.operation)) ||
+	    !ccs_compare_number_union(r->param.inet_network.port, &acl->port))
+		return false;
+	switch (acl->address_type) {
+	case CCS_IP_ADDRESS_TYPE_ADDRESS_GROUP:
+		ret = ccs_address_matches_group(r->param.inet_network.is_ipv6,
+						r->param.inet_network.address,
+						acl->address.group);
+		break;
+	case CCS_IP_ADDRESS_TYPE_IPv4:
+		ret = !r->param.inet_network.is_ipv6 &&
+			acl->address.ipv4.min <= r->param.inet_network.ip &&
+			r->param.inet_network.ip <= acl->address.ipv4.max;
+		break;
+	default:
+		ret = r->param.inet_network.is_ipv6 &&
+			memcmp(acl->address.ipv6.min,
+			       r->param.inet_network.address, 16) <= 0 &&
+			memcmp(r->param.inet_network.address,
+			       acl->address.ipv6.max, 16) <= 0;
+		break;
+	}
+	return ret;
+}
+
+/**
+ * ccs_check_unix_acl - Check permission for unix domain socket operation.
+ *
+ * @r:   Pointer to "struct ccs_request_info".
+ * @ptr: Pointer to "struct ccs_acl_info".
+ *
+ * Returns true if granted, false otherwise.
+ */
+static bool ccs_check_unix_acl(struct ccs_request_info *r,
+			       const struct ccs_acl_info *ptr)
+{
+	const struct ccs_unix_acl *acl = container_of(ptr, typeof(*acl), head);
+	return (acl->perm & (1 << r->param.unix_network.operation)) &&
+		ccs_compare_name_union(r->param.unix_network.address,
+				       &acl->name);
+}
+
 /**
  * ccs_inet_entry - Check permission for INET network operation.
  *
@@ -525,9 +533,6 @@ static int ccs_inet_entry(const struct ccs_addr_info *address)
 	int error = 0;
 	const u8 type = ccs_inet2mac[address->protocol][address->operation];
 	if (type && ccs_init_request_info(&r, type) != CCS_CONFIG_DISABLED) {
-		struct ccs_security * const task = ccs_current_security();
-		const bool no_sleep = address->operation == CCS_NETWORK_ACCEPT
-			|| address->operation == CCS_NETWORK_RECV;
 		r.param_type = CCS_TYPE_INET_ACL;
 		r.param.inet_network.protocol = address->protocol;
 		r.param.inet_network.operation = address->operation;
@@ -538,14 +543,13 @@ static int ccs_inet_entry(const struct ccs_addr_info *address)
 		 * Use host byte order to allow u32 comparison than memcmp().
 		 */
 		r.param.inet_network.ip = ntohl(*address->inet.address);
-		if (no_sleep)
-			task->ccs_flags |= CCS_DONT_SLEEP_ON_ENFORCE_ERROR;
+		r.dont_sleep_on_enforce_error =
+			address->operation == CCS_NETWORK_ACCEPT ||
+			address->operation == CCS_NETWORK_RECV;
 		do {
 			ccs_check_acl(&r, ccs_check_inet_acl);
 			error = ccs_audit_inet_log(&r);
 		} while (error == CCS_RETRY_REQUEST);
-		if (no_sleep)
-			task->ccs_flags &= ~CCS_DONT_SLEEP_ON_ENFORCE_ERROR;
 	}
 	ccs_read_unlock(idx);
 	return error;
@@ -555,7 +559,7 @@ static int ccs_inet_entry(const struct ccs_addr_info *address)
  * ccs_check_inet_address - Check permission for inet domain socket's operation.
  *
  * @addr:     Pointer to "struct sockaddr".
- * @addr_len: Size of @addr .
+ * @addr_len: Size of @addr.
  * @port:     Port number.
  * @address:  Pointer to "struct ccs_addr_info".
  *
@@ -606,21 +610,16 @@ static int ccs_unix_entry(const struct ccs_addr_info *address)
 	int error = 0;
 	const u8 type = ccs_unix2mac[address->protocol][address->operation];
 	if (type && ccs_init_request_info(&r, type) != CCS_CONFIG_DISABLED) {
-		char *buf;
-		if (address->unix0.addr_len <= sizeof(sa_family_t))
-			buf = ccs_encode2("anonymous", 9);
-		else if (address->unix0.addr[0])
-			buf = ccs_encode(address->unix0.addr);
-		else
-			buf = ccs_encode2(address->unix0.addr,
-					  address->unix0.addr_len
-					  - sizeof(sa_family_t));
+		char *buf = address->unix0.addr;
+		int len = address->unix0.addr_len - sizeof(sa_family_t);
+		if (len <= 0) {
+			buf = "anonymous";
+			len = 9;
+		} else if (buf[0]) {
+			len = strnlen(buf, len);
+		}
+		buf = ccs_encode2(buf, len);
 		if (buf) {
-			struct ccs_security * const task =
-				ccs_current_security();
-			const bool no_sleep =
-				address->operation == CCS_NETWORK_ACCEPT ||
-				address->operation == CCS_NETWORK_RECV;
 			struct ccs_path_info addr;
 			addr.name = buf;
 			ccs_fill_path_info(&addr);
@@ -628,16 +627,13 @@ static int ccs_unix_entry(const struct ccs_addr_info *address)
 			r.param.unix_network.protocol = address->protocol;
 			r.param.unix_network.operation = address->operation;
 			r.param.unix_network.address = &addr;
-			if (no_sleep)
-				task->ccs_flags |=
-					CCS_DONT_SLEEP_ON_ENFORCE_ERROR;
+			r.dont_sleep_on_enforce_error =
+				address->operation == CCS_NETWORK_ACCEPT ||
+				address->operation == CCS_NETWORK_RECV;
 			do {
 				ccs_check_acl(&r, ccs_check_unix_acl);
 				error = ccs_audit_unix_log(&r);
 			} while (error == CCS_RETRY_REQUEST);
-			if (no_sleep)
-				task->ccs_flags &=
-					~CCS_DONT_SLEEP_ON_ENFORCE_ERROR;
 			kfree(buf);
 		} else
 			error = -ENOMEM;
@@ -650,7 +646,7 @@ static int ccs_unix_entry(const struct ccs_addr_info *address)
  * ccs_check_unix_address - Check permission for unix domain socket's operation.
  *
  * @addr:     Pointer to "struct sockaddr".
- * @addr_len: Size of @addr .
+ * @addr_len: Size of @addr.
  * @address:  Pointer to "struct ccs_addr_info".
  *
  * Returns 0 on success, negative value otherwise.
@@ -664,14 +660,6 @@ static int ccs_check_unix_address(struct sockaddr *addr,
 		return 0;
 	u->addr = ((struct sockaddr_un *) addr)->sun_path;
 	u->addr_len = addr_len;
-	/*
-	 * Terminate pathname with '\0' like unix_mkname() does.
-	 * This is needed because pathname was copied by move_addr_to_kernel()
-	 * but not yet terminated by unix_mkname().
-	 */
-	if (u->addr[0] && addr_len > sizeof(short) &&
-	    addr_len <= sizeof(struct sockaddr_un))
-		((char *) addr)[addr_len] = '\0';
 	return ccs_unix_entry(address);
 }
 
@@ -765,7 +753,7 @@ static int __ccs_socket_listen_permission(struct socket *sock)
  *
  * @sock:     Pointer to "struct socket".
  * @addr:     Pointer to "struct sockaddr".
- * @addr_len: Size of @addr .
+ * @addr_len: Size of @addr.
  *
  * Returns 0 on success, negative value otherwise.
  */
@@ -801,7 +789,7 @@ static int __ccs_socket_connect_permission(struct socket *sock,
  *
  * @sock:     Pointer to "struct socket".
  * @addr:     Pointer to "struct sockaddr".
- * @addr_len: Size of @addr .
+ * @addr_len: Size of @addr.
  *
  * Returns 0 on success, negative value otherwise.
  */
@@ -833,9 +821,9 @@ static int __ccs_socket_bind_permission(struct socket *sock,
 /**
  * __ccs_socket_sendmsg_permission - Check permission for sending a datagram.
  *
- * @sock:  Pointer to "struct socket".
- * @msg:   Pointer to "struct msghdr".
- * @size:  Unused.
+ * @sock: Pointer to "struct socket".
+ * @msg:  Pointer to "struct msghdr".
+ * @size: Unused.
  *
  * Returns 0 on success, negative value otherwise.
  */
@@ -895,7 +883,7 @@ static int __ccs_socket_post_accept_permission(struct socket *sock,
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
 #if !defined(RHEL_MAJOR) || RHEL_MAJOR != 5
-#if !defined(AX_MAJOR) || AX_MAJOR != 3 || !defined(AX_MINOR) || AX_MINOR < 2
+#if !defined(AX_MAJOR) || AX_MAJOR != 3
 
 /**
  * ip_hdr - Get "struct iphdr".
@@ -944,6 +932,7 @@ static inline struct ipv6hdr *ipv6_hdr(const struct sk_buff *skb)
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+
 /**
  * skb_kill_datagram - Kill a datagram forcibly.
  *
@@ -970,7 +959,9 @@ static inline void skb_kill_datagram(struct sock *sk, struct sk_buff *skb,
 	}
 	skb_free_datagram(sk, skb);
 }
+
 #elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
+
 /**
  * skb_kill_datagram - Kill a datagram forcibly.
  *
@@ -997,7 +988,9 @@ static inline void skb_kill_datagram(struct sock *sk, struct sk_buff *skb,
 	}
 	skb_free_datagram(sk, skb);
 }
+
 #elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
+
 /**
  * skb_kill_datagram - Kill a datagram forcibly.
  *
@@ -1024,6 +1017,7 @@ static inline void skb_kill_datagram(struct sock *sk, struct sk_buff *skb,
 	}
 	skb_free_datagram(sk, skb);
 }
+
 #endif
 
 /**
@@ -1082,12 +1076,13 @@ static int __ccs_socket_post_recvmsg_permission(struct sock *sk,
 				skb->sk->protinfo.af_unix.addr;
 #endif
 			unsigned int addr_len;
-			if (!u)
-				return 0;
-			addr_len = u->len;
-			if (addr_len >= sizeof(addr))
-				return 0;
-			memcpy(&addr, u->name, addr_len);
+			if (u && u->len <= sizeof(addr)) {
+				addr_len = u->len;
+				memcpy(&addr, u->name, addr_len);
+			} else {
+				addr_len = 0;
+				addr.ss_family = AF_UNIX;
+			}
 			if (ccs_check_unix_address((struct sockaddr *) &addr,
 						   addr_len, &address))
 				goto out;
